package com.huigou.topsun.easySearch.controller;

import com.huigou.topsun.resource.service.ResourceFiledService;
import com.huigou.topsun.technology.application.ProcessApplication;
import com.huigou.topsun.technology.domain.query.ProcessQueryRequest;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.bmp.common.easysearch.EasySearch;
import com.huigou.uasp.bmp.common.easysearch.domain.model.EasySearchParse;
import com.huigou.uasp.bmp.common.easysearch.domain.model.QuerySchemeField;
import com.huigou.uasp.bmp.opm.application.MemEasySearcher;
import com.huigou.uasp.bmp.opm.application.impl.MemEasySearcherImpl;
import com.huigou.util.SDO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Controller;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2023/11/24/10:07
 * @Description:
 */
@Controller
@ControllerMapping("/resourceNameGroup")
public class ResourceSearchController {
    @Resource
    private ResourceFiledService resourceFiledService;
    @Resource
    private ProcessApplication processApplication;
    @EasySearch(queryName = "resourceNameGroup")
    public Map<String, Object> queryResourceNameGroup(SDO sdo){
        Integer intPage = sdo.getInteger("intPage", 1);
        Integer pageSize = sdo.getInteger("pageSize",10);
        PageRequest pageRequest = new PageRequest(intPage - 1, pageSize);
        List<QuerySchemeField> fields =
                Arrays.asList(new QuerySchemeField("id", "id", "hidden"),
                        new QuerySchemeField("资源名称", "gxmc", "string", 200L),
                        new QuerySchemeField("资源编码", "gxbm", "string", 100L),
                        new QuerySchemeField("单位", "dw", "string", 100L),
                        new QuerySchemeField("用量", "yl", "string", 100L),
                        new QuerySchemeField("工作中心", "gzzx", "string", 100L));
        EasySearchParse easySearchParse = new EasySearchParse();
        easySearchParse.setFields(fields);
        easySearchParse.setWidth(450L);
        String processType = sdo.getString("paramValue");
        List<Map<String, Object>> mapList = resourceFiledService.getResourceFieldValueListByProcess(processType);
        MemEasySearcher<Map<String, Object>> memEasySearcher = new MemEasySearcherImpl<>();
        Map<String, Object> model = memEasySearcher.search(mapList, easySearchParse, pageRequest, null);
        return model;
    }

    @EasySearch(queryName = "processNameGroup")
    public Map<String, Object> queryProcessNameGroup(SDO sdo){
        Integer intPage = sdo.getInteger("intPage", 1);
        Integer pageSize = sdo.getInteger("pageSize",10);
        PageRequest pageRequest = new PageRequest(intPage - 1, pageSize);
        List<QuerySchemeField> fields =
                Arrays.asList(new QuerySchemeField("processId", "processId", "hidden"),
                        new QuerySchemeField("工序名称", "processName", "string", 200L),
                        new QuerySchemeField("工序类型", "processType", "hidden", 100L),
                        new QuerySchemeField("工序类型", "processTypeTextView", "string", 100L),
                        new QuerySchemeField("工时", "workHours", "string", 100L),
                        new QuerySchemeField("工序内容", "processContent", "hidden", 100L),
                        new QuerySchemeField("工序说明", "processRemark", "hidden", 100L)
                );
        EasySearchParse easySearchParse = new EasySearchParse();
        easySearchParse.setFields(fields);
        easySearchParse.setWidth(450L);
        String processName = sdo.getString("paramValue");
        ProcessQueryRequest queryRequest = sdo.toQueryRequest(ProcessQueryRequest.class);
        queryRequest.setProcessName(processName);
        Map<String, Object> map = processApplication.slicedProcessList(queryRequest);
        List<Map<String, Object>> mapList =  (List<Map<String, Object>>)map.get("Rows");
        MemEasySearcher<Map<String, Object>> memEasySearcher = new MemEasySearcherImpl<>();
        Map<String, Object> model = memEasySearcher.search(mapList, easySearchParse, pageRequest, null);
        return model;
    }
}
