package com.huigou.topsun.proofing.application.impl;

import com.huigou.topsun.product.domain.ProductLooked;
import com.huigou.topsun.product.repository.ProductLookedRepository;
import com.huigou.topsun.proofing.application.ProofingApplyApplication;
import com.huigou.topsun.proofing.domain.model.ProofingApply;
import com.huigou.topsun.proofing.domain.query.ProofingApplyQuery;
import com.huigou.topsun.proofing.domain.vo.ProofingApplyOrLookedVo;
import com.huigou.topsun.proofing.mapper.ProofingApplyMapper;
import com.huigou.topsun.proofing.repository.ProofingApplyRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * @author yonghuan
 */
@Service("proofingApplyApplication")
public class ProofingApplyApplicationImpl extends FlowBroker implements ProofingApplyApplication {

    private ProofingApplyRepository proofingApplyRepository;

    private ProofingApplyMapper proofingApplyMapper;

    private ProductLookedRepository productLookedRepository;

    @Autowired
    public void setProductLookedRepository(ProductLookedRepository productLookedRepository) {
        this.productLookedRepository = productLookedRepository;
    }

    @Autowired
    public void setProofingApplyRepository(ProofingApplyRepository proofingApplyRepository) {
        this.proofingApplyRepository = proofingApplyRepository;
    }

    @Autowired
    public void setProofingApplyMapper(ProofingApplyMapper proofingApplyMapper) {
        this.proofingApplyMapper = proofingApplyMapper;
    }

    @Override
    public ProofingApply loadProofingApplyById(String id) {
        return proofingApplyRepository.findOne(id);
    }

    @Override
    public Map<String, Object> slicedQuery(ProofingApplyQuery query) {
        return proofingApplyMapper.slicedQuery(query);
    }

    @Transactional(propagation = Propagation.REQUIRES_NEW)
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        //从vo类获得两个实体类的属性值
        ProofingApplyOrLookedVo pl=getBizEntity(ProofingApplyOrLookedVo.class);
        ProofingApply proofingApply = new ProofingApply();
        ProductLooked productLooked = new ProductLooked();
        BeanUtils.copyProperties(pl,proofingApply);
        BeanUtils.copyProperties(pl,productLooked);

        if (proofingApply.isNew()) {
            proofingApply.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            proofingApply = (ProofingApply) commonDomainService.loadAndFillinProperties(proofingApply);
        }

        String id= UUID.randomUUID().toString().replace("-", "");
        proofingApply.setProofingApplyId(id);

        proofingApply = proofingApplyRepository.save(proofingApply);
        productLookedRepository.save(productLooked);

        return proofingApply.getId();
    }


    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        // 返回业务数据给流程实例，
        return ClassHelper.toMap(proofingApplyRepository.getOne(bizId));
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        ProofingApply proofingApply = proofingApplyRepository.findOne(bizId);
        proofingApply.setStatusId(status.getId());
        proofingApplyRepository.save(proofingApply);
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        ProofingApply proofingApply = proofingApplyRepository.findOne(bizId);
        proofingApply.setStatusId(BizBillStatus.ABORTED.getId());
        proofingApplyRepository.save(proofingApply);
    }

    /**
     * @param delegateTask
     * @param destActivityId 回退到的目标节点id
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            ProofingApply proofingApply = proofingApplyRepository.findOne(bizId);
            proofingApply.setStatusId(BizBillStatus.APPLYING.getId());
            proofingApplyRepository.save(proofingApply);
        }
    }

    @Override
    public List<ProofingApply> getProofingApplyList() {
        return proofingApplyRepository.findAll();
    }

    @Override
    public ProofingApply getProofingApplyByProofingApplyNo(String proofingApplyNo) {
        return proofingApplyRepository.findByProofingApplyNo(proofingApplyNo);
    }
}
